/*
 * Image mosaic software (C) 1999 Peter Kirchgessner
 * e-mail: peter@kirchgessner.net, WWW: http://www.kirchgessner.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Progress update */
extern void (*msc_progress_function)(double val);
typedef void (* MOSAIC_MSGFUN)(const char *);
extern MOSAIC_MSGFUN msc_message_function;

/* An image structure used for small sized images */
typedef struct
{
 int width, height;
 unsigned char *rgb;
} MOSAIC_IMAGE;


/* Use an image class for large images */
typedef struct mosaic_ic_image_s
{
  void *image;
  void (*img_size_get)(const struct mosaic_ic_image_s *this,
                       int *width, int *height);
  int  (*img_size_set)(struct mosaic_ic_image_s *this,
                       int width, int height);
  void (*img_crop)(const struct mosaic_ic_image_s *this,
                       MOSAIC_IMAGE *img_out,
                       int xpos, int ypos, int width, int height);
  void (*img_paste)(struct mosaic_ic_image_s *this,
                       int xpos, int ypos, const MOSAIC_IMAGE *img);
} MOSAIC_IMAGE_CLASS;


/* Function declarations */
int               msc_files_count (const char *basedir);
int               msc_files_find (const char *basedir, void *userdata,
                                  int (*action)(void *userdata,
                                                const char *subdir,
                                                const char *basename,
                                                int *abort));

int               msc_mia_create (const char *mia_name);
int               msc_images_collect (const char *srcdir, const char *dstdir,
                           int maxsize, int quality,
                           int (*user_read) (const char *filename,
                                             MOSAIC_IMAGE *mi));

/* Functions to handle image class */
void              mscic_init (struct mosaic_ic_image_s *this,
                     void *image,
                     void (*img_size_get)(const MOSAIC_IMAGE_CLASS *this,
                              int *width, int *height),
                     int (*img_size_set)(MOSAIC_IMAGE_CLASS *this,
                              int width, int height),
                     void (*img_crop)(const MOSAIC_IMAGE_CLASS *this,
                              MOSAIC_IMAGE *img_out,
                              int xpos, int ypos, int width,int height),
                     void (*img_paste)(MOSAIC_IMAGE_CLASS *this,
                              int xpos, int ypos, const MOSAIC_IMAGE *img));

int               mscic_mosaic_create (const MOSAIC_IMAGE_CLASS *img_in,
                              const char *archive, int ntiles,
                              int dst_width, int dst_height,
                              int keep_aspect, int exclude_grey,
                              int enable_v_mirror,
                              int enable_random,
                              int reuse_image,
                              MOSAIC_IMAGE_CLASS *img_out);
